<?xml version="1.0" encoding="utf-8"?>

<xsl:stylesheet version="1.0"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns="http://www.w3.org/2000/svg">

	<xsl:output method="xml" version="1.0" indent="yes" encoding="utf-8"/>

	<xsl:param name="unicode"></xsl:param>

	<xsl:template match="@*|*">
		<xsl:copy>
			<xsl:apply-templates select="@*|*"/>
		</xsl:copy>
	</xsl:template>

	<xsl:template match="/">
		<xsl:choose>
			<xsl:when test="$unicode=''">
				<xsl:apply-templates select="//TTGlyph"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="//TTGlyph[@name=
					concat('uni',translate($unicode,'abcdef','ABCDEF'))]"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="TTGlyph">
		<svg>
			<xsl:attribute name="viewBox">
				<xsl:value-of select="concat(@xMin,' ',@yMin,' ',@xMax -@xMin,' ',@yMax -@yMin)"/>
			</xsl:attribute>
			<g transform="matrix(1 0 0 -1 0 {@yMin+@yMax})">
				<xsl:copy-of select="@name"/>
				<xsl:apply-templates select="contour"/>
			</g>
		</svg>
	</xsl:template>

	<xsl:template match="contour">
		<xsl:variable name="pt" select="pt"/>
		<path>
			<xsl:attribute name="d">
				<xsl:value-of select="concat('M',pt[1]/@x,' ',pt[1]/@y)"/>

				<!-- begin loop through points -->
				<xsl:for-each select="pt">

					<xsl:variable name="a">
						<xsl:choose>
							<xsl:when test="position()=count($pt)">
								<xsl:value-of select="1"/>
							</xsl:when>
							<xsl:otherwise>
								<xsl:value-of select="position()+1"/>
							</xsl:otherwise>
						</xsl:choose>
					</xsl:variable>

					<xsl:variable name="b">
						<xsl:choose>
							<xsl:when test="position()=1">
								<xsl:value-of select="0"/>
							</xsl:when>
							<xsl:otherwise>
								<xsl:variable name="i" select="position()-1"/>
								<xsl:value-of select="$pt[position()=$i]/@on"/>
							</xsl:otherwise>
						</xsl:choose>
					</xsl:variable>

					<!-- $pt[$a] is not working -->
					<xsl:call-template name="do_pt">
						<xsl:with-param name="p" select="."/>
						<xsl:with-param name="pa" select="$pt[position()=$a]"/>
						<xsl:with-param name="pb-on" select="$b"/>
					</xsl:call-template>

				</xsl:for-each>

				<!-- process first point if the very last point is Q command -->
				<xsl:if test="$pt[last()]/@on=1 and $pt[1]/@on=0">
					<xsl:call-template name="do_pt">
						<xsl:with-param name="p" select="$pt[1]"/>
						<xsl:with-param name="pa" select="$pt[2]"/>
						<xsl:with-param name="pb-on" select="1"/>
					</xsl:call-template>
				</xsl:if>

				<xsl:text>Z</xsl:text>
			</xsl:attribute>
		</path>
	</xsl:template>

	<xsl:template name="do_pt">
		<xsl:param name="p"/>
		<xsl:param name="pa"/>
		<xsl:param name="pb-on"/>
		<xsl:choose>
			<xsl:when test="$p/@on=1 and $pa/@on=1" >
				<xsl:value-of select="concat('L',$pa/@x,' ',$pa/@y)"/>
			</xsl:when>
			<xsl:when test="$p/@on=0 and $pa/@on=0" >
				<xsl:if test="$pb-on=0">
					<xsl:text>T</xsl:text>
				</xsl:if>
				<xsl:value-of select="concat(0.5*($p/@x+$pa/@x),' ',0.5*($p/@y+$pa/@y))"/>
			</xsl:when>
			<xsl:when test="$p/@on=0 and $pa/@on=1">
				<xsl:if test="$pb-on=0">
					<xsl:text>T</xsl:text>
				</xsl:if>
				<xsl:value-of select="concat($pa/@x,' ',$pa/@y)"/>
			</xsl:when>
			<xsl:when test="$p/@on=1 and $pa/@on=0">
				<xsl:value-of select="concat('Q',$pa/@x,' ',$pa/@y,' ')"/>
			</xsl:when>
		</xsl:choose>
	</xsl:template>

</xsl:stylesheet>
