<?xml version="1.0" encoding="utf-8"?>

<xsl:stylesheet version="1.0"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:str="http://exslt.org/strings"
	xmlns:svg="http://www.w3.org/2000/svg"
	exclude-result-prefixes="svg str"
	xmlns="http://www.w3.org/2000/svg">

	<xsl:output method="xml" version="1.0" indent="yes" encoding="utf-8"/>

	<xsl:param name="order"></xsl:param>
	<xsl:param name="wrap">8</xsl:param>
	<xsl:param name="pen">2</xsl:param>
	<xsl:param name="txt">32</xsl:param>
	<xsl:param name="gap">16</xsl:param>


	<xsl:template match="@*|*">
		<xsl:copy>
			<xsl:apply-templates select="@*|*"/>
		</xsl:copy>
	</xsl:template>


	<xsl:template match="/">
		<xsl:variable name="b" select="str:split(/svg:svg/@viewBox)"/>
		<xsl:variable name="g" select="/svg:svg/svg:g"/>
		<xsl:variable name="total">
			<xsl:choose>
				<xsl:when test="normalize-space($order)=''">
					<xsl:value-of select="count($g/svg:path)"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="count(str:split($order))"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:variable name="extra">
			<xsl:choose>
				<xsl:when test="normalize-space($order)=''">0</xsl:when>
				<xsl:otherwise>
					<xsl:variable name="i" select="$total mod $wrap"/>
					<xsl:choose>
						<xsl:when test="$i=6">2</xsl:when>
						<xsl:when test="$i=7">4</xsl:when>
						<xsl:otherwise>3</xsl:otherwise>
					</xsl:choose>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>

		<svg>
		<xsl:attribute name="viewBox">
			<!-- stroke-width for rect is 2*$pen -->
			<xsl:value-of select="concat('',$b[1] -$gap -$pen)"/>
			<xsl:value-of select="concat(' ',$b[2] -$gap -$pen)"/>
			<xsl:value-of select="concat(' ',2*$pen+($b[3]+2*$gap)*$wrap)"/>
			<xsl:value-of select="concat(' ',2*$pen+($b[4]+2*$gap)*ceiling(($total+$extra) div $wrap))"/>
		</xsl:attribute>

		<rect style="fill:none;stroke-width:{2*$pen};stroke:white;"
			x="{$b[1] -$gap}"
			y="{$b[2] -$gap}"
			width="{($b[3]+2*$gap)*$wrap}"
			height="{($b[4]+2*$gap)*ceiling(($total+$extra) div $wrap)}"/>

		<xsl:for-each select="$g/svg:path">
			<xsl:if test="position() &lt;= $total">
				<g id="s-{position()}-k" style="fill:none;stroke:black;stroke-width:{$pen};">
					<xsl:attribute name="transform">
						<xsl:call-template name="do_matrix">
							<xsl:with-param name="b" select="$b"/>
							<xsl:with-param name="n" select="position()"/>
						</xsl:call-template>
					</xsl:attribute>

					<xsl:if test="normalize-space($order)=''">
						<text x="{$b[1]-$gap+0.25*$txt}" y="{$b[2]-$gap+$txt}" font-size="{$txt}"
							font-family="monospace" style="fill:black;">
							<xsl:value-of select="position()"/>
						</text>
					</xsl:if>

					<xsl:call-template name="do_x">
						<xsl:with-param name="b" select="$b"/>
					</xsl:call-template>

					<xsl:call-template name="do_g">
						<xsl:with-param name="g" select="$g"/>
						<xsl:with-param name="n" select="position()"/>
					</xsl:call-template>
				</g>
			</xsl:if>
		</xsl:for-each>

		<xsl:call-template name="do_extra">
			<xsl:with-param name="total" select="$total"/>
			<xsl:with-param name="b" select="$b"/>
			<xsl:with-param name="n" select="$extra"/>
		</xsl:call-template>
		</svg>
	</xsl:template>


	<xsl:template name="do_extra">
		<xsl:param name="total"/>
		<xsl:param name="b"/>
		<xsl:param name="n"/>
		<xsl:if test="$n &gt; 0">
			<g id="s-{$total+$n}-q" style="fill:none;stroke:black;stroke-width:{$pen};">
				<xsl:attribute name="transform">
					<xsl:call-template name="do_matrix">
						<xsl:with-param name="b" select="$b"/>
						<xsl:with-param name="n" select="$total+$n"/>
					</xsl:call-template>
				</xsl:attribute>

				<xsl:call-template name="do_x">
					<xsl:with-param name="b" select="$b"/>
				</xsl:call-template>
			</g>

			<xsl:call-template name="do_extra">
				<xsl:with-param name="total" select="$total"/>
				<xsl:with-param name="b" select="$b"/>
				<xsl:with-param name="n" select="$n -1"/>
			</xsl:call-template>
		</xsl:if>
	</xsl:template>


	<xsl:template name="do_matrix">
		<xsl:param name="b"/>
		<xsl:param name="n"/>
		<xsl:text>translate(</xsl:text>
		<xsl:value-of select="($b[3]+2*$gap)*(($n -1) mod $wrap)"/>
		<xsl:text> </xsl:text>
		<xsl:value-of select="($b[4]+2*$gap)*floor(($n -1) div $wrap)"/>
		<xsl:text>)</xsl:text>
	</xsl:template>


	<xsl:template name="do_x">
		<xsl:param name="b"/>
	 	<xsl:variable name="d" select="round(($b[3]+2*$gap) div 17)"/>
		<rect style="stroke-width:{2*$pen};"
			x="{$b[1]-$gap}" y="{$b[2]-$gap}"
			width="{$b[3]+2*$gap}" height="{$b[4]+2*$gap}"/>
		<line stroke-dasharray="{$d} {$d}"
			x1="{$b[1]-$gap}" y1="{$b[2]+0.5*$b[4]}"
			x2="{$b[1]+$b[3]+$gap}" y2="{$b[2]+0.5*$b[4]}"/>
		<line stroke-dasharray="{$d} {$d}"
			x1="{$b[1]+0.5*$b[3]}" y1="{$b[2]-$gap}"
			x2="{$b[1]+0.5*$b[3]}" y2="{$b[2]+$b[4]+$gap}"/>
	</xsl:template>


	<xsl:template name="do_g">
		<xsl:param name="g"/>
		<xsl:param name="n"/>
		<g>
		<xsl:apply-templates select="$g/@*"/>

		<xsl:choose>
			<xsl:when test="normalize-space($order)=''">
				<xsl:for-each select="$g/svg:path[position()!=$n]">
					<xsl:call-template name="do_p">
						<xsl:with-param name="p" select="."/>
						<xsl:with-param name="c" select="'gray'"/>
					</xsl:call-template>
				</xsl:for-each>
				<xsl:call-template name="do_p">
					<xsl:with-param name="p" select="$g/svg:path[$n]"/>
					<xsl:with-param name="c" select="'black'"/>
				</xsl:call-template>
			</xsl:when>

			<xsl:otherwise>
				<xsl:for-each select="str:split($order)[position() &lt;= $n]">
				<xsl:for-each select="str:split(.,'-')">
					<!-- Somehow it won't work if put number(.) in the place $i -->
					<xsl:variable name="i" select="number(.)"/>
					<xsl:call-template name="do_p">
						<xsl:with-param name="p" select="$g/svg:path[$i]"/>
						<xsl:with-param name="c" select="'black'"/>
					</xsl:call-template>
				</xsl:for-each>
				</xsl:for-each>
			</xsl:otherwise>
		</xsl:choose>
		</g>
	</xsl:template>


	<xsl:template name="do_p">
		<xsl:param name="p"/>
		<xsl:param name="c"/>
		<path style="fill:{$c};stroke:{$c}">
			<xsl:apply-templates select="$p/@*|$p/*"/>
		</path>
	</xsl:template>

</xsl:stylesheet>
